@@ -1,6 +1,6 @@
 /*
   Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2019-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -98,6 +98,7 @@ int grn_uyield_count = 0;
 #endif
 
 static grn_bool grn_ctx_per_db = GRN_FALSE;
+static bool grn_back_trace_enable = true;
 
 static void
 grn_init_from_env(void)
@@ -112,6 +113,17 @@ grn_init_from_env(void)
     }
   }
 
+  {
+    char grn_back_trace_enable_env[GRN_ENV_BUFFER_SIZE];
+    grn_getenv("GRN_BACK_TRACE_ENABLE",
+               grn_back_trace_enable_env,
+               GRN_ENV_BUFFER_SIZE);
+    if (grn_back_trace_enable_env[0] &&
+        strcmp(grn_back_trace_enable_env, "no") == 0) {
+      grn_back_trace_enable = false;
+    }
+  }
+
   grn_alloc_init_from_env();
   grn_mrb_init_from_env();
   grn_ctx_impl_mrb_init_from_env();
@@ -2022,12 +2034,15 @@ grn_ctx_logv(grn_ctx *ctx, const char *fmt, va_list ap)
 }
 
 #ifdef WIN32
-static void
+static __declspec(noinline) void
 grn_ctx_log_back_trace_windows(grn_ctx *ctx, grn_log_level level)
 {
+# ifdef GRN_WITH_WINDOWS_BACK_TRACE
+#  define MAX_N_FRAMES 62
   ULONG frames_to_skip = 0;
-  ULONG frames_to_capture = MAXUSHORT;
-  void *back_trace[MAXUSHORT];
+  ULONG frames_to_capture = MAX_N_FRAMES;
+  void *back_trace[MAX_N_FRAMES];
+#  undef MAX_N_FRAMES
   USHORT n_frames = CaptureStackBackTrace(frames_to_skip,
                                           frames_to_capture,
                                           back_trace,
@@ -2051,6 +2066,7 @@ grn_ctx_log_back_trace_windows(grn_ctx *ctx, grn_log_level level)
   }
 
   grn_windows_symbol_cleanup(ctx, process);
+# endif
 }
 #else
 # ifdef HAVE_BACKTRACE
@@ -2076,9 +2092,25 @@ grn_ctx_log_back_trace_backtrace(grn_ctx *ctx, grn_log_level level)
 # endif
 #endif
 
+bool
+grn_is_back_trace_enable(void)
+{
+  return grn_back_trace_enable;
+}
+
+grn_rc
+grn_set_back_trace_enable(bool enable)
+{
+  grn_back_trace_enable = enable;
+  return GRN_SUCCESS;
+}
+
 void
 grn_ctx_log_back_trace(grn_ctx *ctx, grn_log_level level)
 {
+  if (!grn_back_trace_enable) {
+    return;
+  }
 #ifdef WIN32
   grn_ctx_log_back_trace_windows(ctx, level);
 #else
